<?php

namespace App\Core\Elastic\Product;

use App\Core\Elastic\ElasticQuery;
use App\Core\Elastic\Extractor;
use App\Core\Elastic\StagedIndex;
use Elasticsearch\Client;
use Illuminate\Support\Collection;
use Psr\Log\LoggerInterface;

/**
 * Класс для работы с индексом товаров в Эластике.
 *
 * Class ProductIndex
 * @package App\Core\Elastic\Product
 */
class ProductIndex extends StagedIndex
{
    public const TAG_NEW = 'new';

    /** @var string Имя индекса (по факту префиксом к нему идет среда: например dev_products) */
    protected $name = 'products';

    /** @var Client */
    private $client;

    /** @var LoggerInterface */
    protected $logger;

    /**
     * ProductIndex constructor.
     *
     * @param Client $client
     */
    public function __construct(Client $client)
    {
        $this->client = $client;
    }

    /**
     * Удаляет индекс товаров из Эластика.
     */
    public function down(): void
    {
        $this->client->indices()->delete(['index' => $this->fullIndexName()]);
    }

    /**
     * Создает индекс товаров в Эластике.
     */
    public function up(): void
    {
        $this->client->indices()->create([
            'index' => $this->fullIndexName(),
            'body' => [
                'settings' => [
                    'number_of_shards' => 1,
                    'index.number_of_replicas' => '0',
                    'index.max_result_window' => ElasticQuery::MAX_LIMIT,
                    'analysis' => [
                        "char_filter"=> [
                            "rus_en_key" => [
                                "type" => "mapping",
                                "mappings" => [
                                    "a => ф",
                                    "b => и",
                                    "c => с",
                                    "d => в",
                                    "e => у",
                                    "f => а",
                                    "g => п",
                                    "h => р",
                                    "i => ш",
                                    "j => о",
                                    "k => л",
                                    "l => д",
                                    "m => ь",
                                    "n => т",
                                    "o => щ",
                                    "p => з",
                                    "r => к",
                                    "s => ы",
                                    "t => е",
                                    "u => г",
                                    "v => м",
                                    "w => ц",
                                    "x => ч",
                                    "y => н",
                                    "z => я",
                                    "A => Ф",
                                    "B => И",
                                    "C => С",
                                    "D => В",
                                    "E => У",
                                    "F => А",
                                    "G => П",
                                    "H => Р",
                                    "I => Ш",
                                    "J => О",
                                    "K => Л",
                                    "L => Д",
                                    "M => Ь",
                                    "N => Т",
                                    "O => Щ",
                                    "P => З",
                                    "R => К",
                                    "S => Ы",
                                    "T => Е",
                                    "U => Г",
                                    "V => М",
                                    "W => Ц",
                                    "X => Ч",
                                    "Y => Н",
                                    "Z => Я",
                                    "[ => х",
                                    "] => ъ",
                                    "; => ж",
                                    "< => б",
                                    "> => ю",
                                ],
                            ],
                        ],
                        'filter' => [
                            'russian_stop' => [
                                'type' => 'stop',
                                'stopwords' => '_russian_',
                            ],
                            'russian_stemmer' => [
                                'type' => 'stemmer',
                                'language' => 'russian',
                            ],
                        ],
                        'analyzer' => [
                            'default' => [
                                'type' => 'custom',
                                "char_filter" => ['html_strip', 'rus_en_key'],
                                'tokenizer' => 'standard',
                                'filter' => ['lowercase', 'russian_stop', 'russian_stemmer'],
                            ],
                        ],
                    ],
                ],
                'mappings' => [
                    'properties' => [
                        'active' => ['type' => 'boolean'],
                        'archive' => ['type' => 'keyword'],

                        'has_active_offers' => ['type' => 'boolean'],

                        'name' => ['type' => 'text'],
                        'code' => ['type' => 'keyword'],
                        'barcode' => ['type' => 'keyword'],
                        'external_id' => ['type' => 'keyword'],

                        'offers' => [
                            'type' => 'nested',
                            'properties' => [
                                'active' => ['type' => 'boolean'],
                                'saleActive' => ['type' => 'boolean'],

                                'offer_id' => ['type' => 'keyword'],
                                'seller_id' => ['type' => 'keyword'],

                                'price' => [
                                    'type' => 'object',
                                ],

                                'qty' => ['type' => 'integer'],
                                'store_id' => ['type' => 'keyword', 'index' => false],
                            ],
                        ],

                        'brand' => [
                            'properties' => [
                                'code' => ['type' => 'keyword'],
                                'name' => ['type' => 'keyword'],
                            ],
                        ],
                        'category' => [
                            'properties' => [
                                'code' => ['type' => 'keyword'],
                                'name' => ['type' => 'keyword'],
                            ],
                        ],
                        'tags' => ['type' => 'keyword'],
                        'rating' => ['type' => 'keyword'],
                        'preorder' => ['type' => 'boolean'],
                        'props' => ['type' => 'keyword'],
                        'product_groups' => ['type' => 'keyword'],

                        'date_add' => ['type' => 'date'],

                        'catalog_image' => ['type' => 'keyword', 'index' => false],

                        'description' => [
                            'properties' => [
                                'text' => ['type' => 'text'],
                                'video' => ['type' => 'keyword', 'index' => false],
                                'image' => ['type' => 'keyword', 'index' => false],
                            ],
                        ],
                        'media' => ['type' => 'keyword', 'index' => false],
                        'variant_group' => ['type' => 'keyword', 'index' => false],

                        'characteristics' => [
                            'properties' => [
                                'name' => ['type' => 'text', 'index' => false],
                                'value' => ['type' => 'text', 'index' => false],
                            ],
                        ],
                        'strikes' => ['type' => 'text', 'index' => false],
                    ],
                ],
            ],
        ]);
    }

    /**
     * Индекс товаров существует?
     *
     * @return bool
     */
    public function isUp(): bool
    {
        return $this->client->indices()->exists(['index' => $this->fullIndexName()]);
    }

    /**
     * Добавить товар в индекс
     * @param $id
     * @param ProductIndexRecord $product
     */
    public function upsert($id, ProductIndexRecord $product): void
    {
        $this->client->index([
            'index' => $this->fullIndexName(),
            'id' => $id,
            'body' => $product->toArray(),
        ]);
    }

    /**
     * Убрать товар из индекса
     * @param int $id
     */
    public function delete(int $id): void
    {
        $this->client->delete([
            'index' => $this->fullIndexName(),
            'id' => $id,
        ]);
    }

    /**
     * Выполняет пакет операций.
     * @param Collection|ProductIndexOperation[] $operations
     */
    public function bulk(Collection $operations): void
    {
        $indexName = $this->fullIndexName();
        $body = $operations->flatMap(fn (ProductIndexOperation $oper) => $oper->encode($indexName))->toArray();

        $this->client->bulk(['body' => $body]);
    }

    /**
     * @param ElasticQuery $query
     * @param Extractor $extractor
     * @return array
     */
    public function search(ElasticQuery $query, Extractor $extractor)
    {
        $response = $this->client->search([
            'index' => $this->fullIndexName(),
            'body' => $query->query(),
            'track_total_hits' => true,
        ]);

        $result = [];
        if ($response['hits']['hits']) {
            foreach ($response['hits']['hits'] as $hit) {
                $result[] = $extractor->extract($hit);
            }
        }
        $aggregations = $extractor->extractAggregation($response);

        return [
            'total' => $response['hits']['total']['value'],
            'items' => $result,
            'aggregations' => $aggregations,
        ];
    }
}
