<?php

namespace App\Core\Elastic\Product;

use App\Core\Elastic\Indexer;
use App\Domain\Products\Models\Product;
use App\Domain\Products\Models\ProductImage;
use App\Models\Offer;
use App\Models\SaleStatus;
use Ensi\CmsClient\Api\ProductGroupsApi;
use Ensi\CmsClient\Dto\GetGroupsByProductsRequest;
use Ensi\LaravelEnsiFilesystem\Models\EnsiFile;
use Ensi\MarketingClient\Api\DiscountsApi;
use Ensi\MarketingClient\Dto\CalculateForCatalogRequest;
use Ensi\MarketingClient\Dto\DiscountOfferCatalogRequest;
use Ensi\OffersClient\Api\StocksApi;
use Ensi\OffersClient\Dto\Stock;
use Illuminate\Database\Eloquent\Collection;

/**
 * Class ProductIndexer
 * @package App\Core\Elastic\Product
 */
class ProductIndexer implements Indexer
{
    private const MIN_STOCK = 4;

    /** @var ProductIndex */
    private $index;
    /** @var StocksApi */
    private $stocksApi;
    /** @var DiscountsApi */
    private $discountsApi;

    /** @var ProductGroupsApi */
    private $productGroupsApi;

    /** @var Collection|Product[] */
    private $products;
    /** @var array */
    private $offersByProducts;
    /** @var array */
    private $prices;
    /** @var array */
    private $productGroups;

    /**
     * ProductIndexer constructor.
     * @param ProductIndex $index
     */
    public function __construct(ProductIndex $index)
    {
        $this->index = $index;
        $this->stocksApi = resolve(StocksApi::class);
        $this->discountsApi = resolve(DiscountsApi::class);
        $this->productGroupsApi = resolve(ProductGroupsApi::class);
    }

    /**
     * @param array $ids
     */
    public function preload(array $ids)
    {
        $this->products = Product::query()
            ->with(['tips', 'brand', 'category', 'properties', 'properties.property', 'properties.propertyDirectoryValue'])
            ->whereIn('id', $ids)
            ->get()
            ->keyBy('id');
        $this->offersByProducts = Offer::getActiveByProducts($ids);

        $offerIds = [];
        foreach ($this->offersByProducts as $productId => $offers) {
            /**
             * @var int $sellerId
             * @var Offer $offer
             */
            foreach ($offers as $sellerId => $offer) {
                $offerIds[$offer->id] = $offer->id;
            }
        }

        if ($offerIds) {
            $this->fetchCatalogPrices();

            $getGroupsByProductsRequest = new GetGroupsByProductsRequest();
            $getGroupsByProductsRequest->setProductIds($ids);
            $this->productGroups = $this->productGroupsApi
                ->getGroupsByProducts($getGroupsByProductsRequest)
                ->getProductGroupIds();
        }
    }

    public function index(int $id): bool
    {
        /** @var Product $product */
        $product = $this->products->get($id);
        if (!$product) {
            try {
                $this->index->delete($id);
            } catch (\Exception $e) {
            }

            return true;
        }

        $productRecord = new ProductIndexRecord();
        $this->addBaseInfo($product, $productRecord);
        $this->addPropsInfo($product, $productRecord);
        $this->addImagesInfo($product, $productRecord);
        $this->addOfferInfo($id, $productRecord);

        // throw new \Exception(json_encode($product->images->toArray()));
        $this->index->upsert($product->id, $productRecord);

        return true;
    }

    /**
     * @param Product $product
     * @param ProductIndexRecord $productRecord
     */
    private function addBaseInfo(Product $product, ProductIndexRecord $productRecord)
    {
        $productRecord->name = $product->name;
        $productRecord->code = $product->code;
        $productRecord->barcode = $product->barcode;
        $productRecord->externalId = $product->external_id;

        if ($product->is_new) {
            $productRecord->isNew = true;
            $productRecord->tags[] = ProductIndex::TAG_NEW;
        }

        $productRecord->archive = $product->archive->value;

        $productRecord->dateAdd = $product->created_at;

        $productRecord->weight = $product->weight_gross;

        $brand = $product->brand;
        if ($brand) {
            $productRecord->brandName = $brand->name;
            $productRecord->brandCode = $brand->code;
        }
        $category = $product->category;
        if ($category) {
            $productRecord->categoryName = $category->name;
            $codes = $category->ancestors->pluck('code')->all();
            $codes[] = $category->code;
            $productRecord->categoryCodes = $codes;
        } else {
            $productRecord->addStrike("Нет категории");
        }
        if (isset($this->productGroups[$product->id])) {
            $productRecord->groups = $this->productGroups[$product->id];
        }
        $productRecord->variantGroupId = $product->variant_group_id;

        if ($product->archive->isArchive()) {
            $productRecord->addStrike('В архиве');
        }
    }

    /**
     * @param Product $product
     * @param ProductIndexRecord $productRecord
     */
    private function addPropsInfo(Product $product, ProductIndexRecord $productRecord)
    {
        $characteristics = [];
        foreach ($product->properties as $propertyValue) {
            $property = $propertyValue->property;
            if ($propertyValue->propertyDirectoryValue) {
                $propValue = $propertyValue->propertyDirectoryValue->code;
                $propVisibleValue = $propertyValue->propertyDirectoryValue->name;
            } else {
                $propValue = $propertyValue->value($property->type);
                $propVisibleValue = $propValue;
            }
            if ($property->is_filterable) {
                $productRecord->props[] = "{$property->code}#{$propValue}";
            }
            if (!isset($characteristics[$property->name])) {
                $characteristics[$property->name] = [];
            }
            $characteristics[$property->name][] = $propVisibleValue;
        }

        $this->fillCharacteristics($product, $characteristics);

        foreach ($characteristics as $name => $values) {
            $productRecord->addCharacteristic($name, $values);
        }
    }

    /**
     * Обогащает характеристики данными из товара.
     *
     * @param Product $product
     * @param array $characteristics
     */
    protected function fillCharacteristics(Product $product, array &$characteristics)
    {
        if ($product->calories) {
            $characteristics['Калорийность'][] = $product->calories;
        }
        if ($product->proteins) {
            $characteristics['Белки на 100 г, г'][] = $product->proteins;
        }
        if ($product->fats) {
            $characteristics['Жиры на 100 г, г'][] = $product->fats;
        }
        if ($product->carbohydrates) {
            $characteristics['Углеводы на 100 г, г'][] = $product->carbohydrates;
        }
        if ($product->weight) {
            $characteristics['Масса нетто, кг'][] = $product->weight;
        }
        if ($product->weight_gross) {
            $characteristics['Масса брутто, кг'][] = $product->weight_gross;
        }
        if ($product->ingredients) {
            $characteristics['Ингредиенты'][] = $product->ingredients;
        }
        if ($product->length || $product->width || $product->height) {
            $characteristics['ДxШxВ, мм'][] = ($product->length ?? '-') . 'x' . ($product->width ?? '-') . 'x' . ($product->height ?? '-');
        }
        if ($product->manufacturer) {
            $characteristics['Производитель'][] = $product->manufacturer->name;
        }
        if ($product->country) {
            $characteristics['Страна производства'][] = $product->country->name;
        }
    }

    /**
     * @param Product $product
     * @param ProductIndexRecord $productRecord
     */
    private function addImagesInfo(Product $product, ProductIndexRecord $productRecord)
    {
        /** @var Collection|ProductImage[] $images */
        $images = $product->images->filter(function ($image) {
            return $image->file;
        })->groupBy('code');
        if ($images->has(ProductImage::TYPE_MAIN)) {
            /** @var ProductImage $catalogImage */
            $catalogImage = $images->get(ProductImage::TYPE_MAIN)->first();
            $productRecord->catalogImage = EnsiFile::public($catalogImage->file)->getUrl();
        } else {
            $productRecord->addStrike('Нет изображения для каталога');
        }

        if ($images->has(ProductImage::TYPE_GALLERY)) {
            /** @var ProductImage $galleryImage */
            foreach ($images->get(ProductImage::TYPE_GALLERY) as $galleryImage) {
                $productRecord->media[] = EnsiFile::public($galleryImage->file)->getUrl();
            }
        }

        $descriptionImageId = null;
        if ($images->has(ProductImage::TYPE_DESCRIPTION)) {
            /** @var ProductImage $descriptionImage */
            $descriptionImage = $images->get(ProductImage::TYPE_DESCRIPTION)->first();
            $descriptionImageId = EnsiFile::public($descriptionImage->file)->getUrl();
        }

        $productRecord->description($product->description, $descriptionImageId, $product->description_video);
    }

    /**
     * @param int $id
     * @param ProductIndexRecord $productRecord
     * @throws \Exception
     */
    private function addOfferInfo(int $id, ProductIndexRecord $productRecord)
    {
        $offers = $this->offersByProducts[$id] ?? [];
        $stocks = collect($this->stocksApi->getStocksByProduct($id)->getItems())->keyBy('offer_id');

        if (!$offers) {
            $productRecord->addStrike('Нет активного оффера');
        }

        /**
         * @var int $sellerId
         * @var Offer $offer
         */
        foreach ($offers as $sellerId => $offer) {
            /** @var Stock $stock */
            $stock = $stocks[$offer->id] ?? null;

            $productRecord->offer($sellerId)->sellerId = $sellerId;

            if ($offer && $offer->id) {
                $productRecord->offer($sellerId)->offerId = $offer->id;
            }

            if ($offer && $offer->storage_address) {
                $productRecord->offer($sellerId)->storageAddress = $offer->storage_address;
            }

            if ($stock && $stock->getStoreId()) {
                $productRecord->offer($sellerId)->storeId = $stock->getStoreId();
            } else {
                $productRecord->addOfferStrike($sellerId, 'Нет склада');
            }

            if ($stock) {
                $productRecord->offer($sellerId)->qty = $stock->getQty();
            }
            if ($stock && $stock->getQty() < static::MIN_STOCK) {
                $productRecord->addOfferStrike($sellerId, "Сток товара на складе меньше " . static::MIN_STOCK);
            }

            if (!SaleStatus::onSale($offer->sale_status)) {
                $productRecord->resetSaleActive($sellerId);
            }

            $price = $this->prices[$offer->id] ?? null;
            if ($price) {
                $productRecord->addPrice($sellerId, [
                    'cost' => $price->getCost(),
                    'price' => $price->getPrice(),
                ]);
            } else {
                $productRecord->addOfferStrike($sellerId, 'Нет цены');
            }
        }
    }

    /**
     * @todo implement receiving prices for offers
     */
    private function fetchCatalogPrices()
    {
        $reqOffers = [];
        $request = new CalculateForCatalogRequest();
        foreach ($this->offersByProducts as $productId => $offers) {
            $product = $this->products[$productId];
            foreach ($offers as $offer) {
                $reqOffer = (new DiscountOfferCatalogRequest())
                    ->setOfferId($offer->id)
                    ->setProductId($productId)
                    ->setBrandId($product['brand_id'])
                    ->setCategoryId($product['category_id'])
                    ->setPrice(1000); // todo: receiving price for offer

                $reqOffers[] = $reqOffer;
            }
        }

        $request->setOffers($reqOffers);
        $items = $this->discountsApi->calculateForCatalog($request)->getData();

        $this->prices = [];
        foreach ($items as $item) {
            $this->prices[$item->getOfferId()] = $item;
        }
    }
}
