<?php

namespace App\Core\Elastic;

use App\Domain\Products\Models\IndexRequest;
use App\Support\Log\DefaultLoggerAwareInterface;
use Exception;
use Psr\Log\LoggerAwareTrait;

class ScheduleRunner implements DefaultLoggerAwareInterface
{
    use LoggerAwareTrait;

    public function markForIndex(string $type, int $id): bool
    {
        /** @var IndexRequest $indexRequest */
        $indexRequest = IndexRequest::query()
            ->where('type', $type)
            ->where('object_id', $id)
            ->first();

        if ($indexRequest) {
            $indexRequest->strike = false;
        } else {
            $indexRequest = new IndexRequest();
            $indexRequest->type = $type;
            $indexRequest->object_id = $id;
        }

        return $indexRequest->save();
    }

    public function checkMarked(string $type, array $ids): array
    {
        $requests = IndexRequest::query()
            ->where('type', $type)
            ->whereIn('object_id', $ids)
            ->get();

        return $requests->pluck('object_id')->all();
    }

    public function workOnMarked(IndexerFactory $factory)
    {
        IndexRequest::query()
            // ->where('strike', false)
            ->orderBy('id')
            ->chunk(100, function ($indexRequests) use ($factory) {
                $objectsByType = [];

                foreach ($indexRequests as $indexRequest) {
                    if (!isset($objectsByType[$indexRequest->type])) {
                        $objectsByType[$indexRequest->type] = [];
                    }
                    $objectsByType[$indexRequest->type][] = $indexRequest->object_id;
                }

                foreach ($objectsByType as $type => $ids) {
                    $indexer = $factory->indexer($type);
                    if (!$indexer) {
                        continue;
                    }

                    try {
                        $indexer->preload($ids);
                    } catch (Exception $e) {
                        $this->logger?->error("Ошибка индексирования блока товаров: {$e->getMessage()}");
                    }

                    foreach ($ids as $id) {
                        try {
                            $ok = $indexer->index($id);
                        } catch (Exception $e) {
                            $ok = false;
                            $this->logger?->debug("Ошибка индексирования товара $id: {$e->getMessage()}");
                        }

                        $query = IndexRequest::query()
                            ->where('type', $type)
                            ->where('object_id', $id);

                        if (!$ok) {
                            $query->update(['strike' => true]);
                        } else {
                            $query->update(['delete' => true]);
                        }
                    }
                }

                return true;
            });

        IndexRequest::query()->where('delete', true)->delete();
    }
}
