<?php

namespace App\Domain\Products\Models;

use App\Domain\Products\Models\Tests\Factories\ProductImageFactory;
use App\Domain\Support\Concerns\SupportsPublishing;
use App\Domain\Support\Models\Model;
use Carbon\CarbonInterface;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;

/**
 * ProductImage картинка товара.
 *
 * @property int $id Идентификатор картинки
 * @property int $product_id Идентификатор товара
 * @property int $sort Порядок сортировки
 * @property string|null $file Файл картинки
 * @property string|null $url URL внешнего файла
 * @property string|null $name Описание картинки
 * @property CarbonInterface|null $deleted_at Время пометки на удаление
 *
 * @property PublishedImage|null $release Опубликованная картинка
 * @property Product $product Товар, которому принадлежит изображение
 *
 * @method static Builder|static whereProductId(int $id)
 */
class ProductImage extends Model
{
    use SupportsPublishing;

    const FILLABLE = ['sort', 'name'];

    protected $fillable = self::FILLABLE;

    protected $casts = [
        'product_id' => 'int',
        'sort' => 'int',
    ];

    protected $attributes = [
        'sort' => 100,
    ];

    public function isExternal(): bool
    {
        return filled($this->url);
    }

    public function isInternal(): bool
    {
        return filled($this->file);
    }

    public function isValid(): bool
    {
        return $this->exists && !$this->trashed();
    }

    protected function newRelease(): Model
    {
        $release = new PublishedImage();

        $release->product_id = $this->product_id;
        $release->file = $this->file;
        $release->url = $this->url;

        return $release;
    }

    public function product(): BelongsTo
    {
        return $this->belongsTo(Product::class, 'product_id');
    }

    public function release(): BelongsTo
    {
        return $this->belongsTo(PublishedImage::class, 'id', 'id');
    }

    public static function factory(array $attributes = []): ProductImageFactory
    {
        return ProductImageFactory::new($attributes);
    }
}
