<?php

namespace Tests\Integration\Domain\Support\Pagination;

use App\Http\ApiV1\Support\Pagination\CursorPageBuilder;
use Illuminate\Pagination\CursorPaginator;
use Illuminate\Support\Collection;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use UnexpectedValueException;

class CursorPageBuilderTest extends AbstractPaginationTest
{
    public function testPositiveLimit(): void
    {
        $limit = 10;
        $request = $this->cpbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $expectedItems = [1, 2, 3, 4];
        $queryBuilderMock->shouldReceive('cursorPaginate')->andReturn(new CursorPaginator($expectedItems, $limit));
        $builder = new CursorPageBuilder($queryBuilderMock, $request);

        $page = $builder->build();

        expect($page->items)->toMatchArray($expectedItems);
        expect($page->pagination)->toMatchArray([
            "cursor" => null,
            "limit" => $limit,
            "next_cursor" => null,
            "previous_cursor" => null,
            "type" => "cursor",
        ]);
    }

    public function testPaginationFailed()
    {
        $limit = 10;
        $request = $this->cpbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);

        $this->expectException(BadRequestHttpException::class);

        $queryBuilderMock = $this->mockQueryBuilder();
        $queryBuilderMock->shouldReceive('cursorPaginate')->andThrow(new UnexpectedValueException("test"));
        $builder = new CursorPageBuilder($queryBuilderMock, $request);
        $builder->build();
    }

    public function testRequestCursorNotDecoded()
    {
        $limit = 10;
        $request = $this->cpbMakeRequest([
            'pagination' => [
                'limit' => $limit,
                'cursor' => 'foo',
            ],
        ]);

        $this->expectException(BadRequestHttpException::class);

        $queryBuilderMock = $this->mockQueryBuilder();
        $builder = new CursorPageBuilder($queryBuilderMock, $request);

        $builder->build();
    }

    public function testLimitReturnEmptyArray()
    {
        $limit = 0;
        $request = $this->cpbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $builder = new CursorPageBuilder($queryBuilderMock, $request);

        $page = $builder->build();

        expect($page->items)->toMatchArray([]);
        expect($page->pagination)->toMatchArray([
            "cursor" => null,
            "limit" => $limit,
            "next_cursor" => null,
            "previous_cursor" => null,
            "type" => "cursor",
        ]);
    }

    public function testNegativeLimit()
    {
        $limit = -1;
        $request = $this->cpbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $expectedItems = [1, 2, 3, 4];
        $queryBuilderMock->shouldReceive('get')->andReturn(new Collection($expectedItems));
        $builder = new CursorPageBuilder($queryBuilderMock, $request);

        $page = $builder->build();

        expect($page->items)->toMatchArray($expectedItems);
        expect($page->pagination)->toMatchArray([
            "cursor" => null,
            "limit" => $limit,
            "next_cursor" => null,
            "previous_cursor" => null,
            "type" => "cursor",
        ]);
    }

    public function testNegativeLimitAndForbidToBypassPagination()
    {
        $limit = -1;
        $request = $this->cpbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $builder = new CursorPageBuilder($queryBuilderMock, $request);

        $page = $builder->forbidToBypassPagination()->build();

        expect($page->items)->toMatchArray([]);
        expect($page->pagination)->toMatchArray([
            "cursor" => null,
            "limit" => $limit,
            "next_cursor" => null,
            "previous_cursor" => null,
            "type" => "cursor",
        ]);
    }

    public function testPositiveLimitCannotExceedMaxLimit()
    {
        $limit = 10;
        $maxLimit = 5;
        $request = $this->cpbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $expectedItems = [1, 2, 3, 4];
        $queryBuilderMock->shouldReceive('cursorPaginate')->andReturn(new CursorPaginator($expectedItems, $maxLimit));
        $builder = new CursorPageBuilder($queryBuilderMock, $request);

        $page = $builder->maxLimit($maxLimit)->build();

        expect($page->items)->toMatchArray($expectedItems);
        expect($page->pagination)->toMatchArray([
            "cursor" => null,
            "limit" => $maxLimit,
            "next_cursor" => null,
            "previous_cursor" => null,
            "type" => "cursor",
        ]);
    }
}
