<?php

namespace Tests\Integration\Domain\Support\Pagination;

use App\Http\ApiV1\Support\Pagination\OffsetPageBuilder;
use Illuminate\Support\Collection;

class OffsetPageBuilderTest extends AbstractPaginationTest
{
    public function testPositiveLimit()
    {
        $limit = 10;
        $request = $this->ofbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $expectedItems = [1, 2, 3, 4];
        $queryBuilderMock->shouldReceive('clone')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('skip')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('limit')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('get')->andReturn(new Collection($expectedItems));
        $builder = new OffsetPageBuilder($queryBuilderMock, $request);

        $page = $builder->build();

        expect($page->items)->toMatchArray($expectedItems);
        expect($page->pagination)->toMatchArray([
            "offset" => 0,
            "limit" => $limit,
            "total" => 4,
            "type" => "offset",
        ]);
    }

    public function testNeedsAnotherCountQuery()
    {
        $limit = 4;
        $total = 12;
        $request = $this->ofbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $expectedItems = [1, 2, 3, 4];
        $queryBuilderMock->shouldReceive('clone')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('skip')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('limit')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('get')->andReturn(new Collection($expectedItems));
        $queryBuilderMock->shouldReceive('count')->andReturn($total);
        $builder = new OffsetPageBuilder($queryBuilderMock, $request);

        $page = $builder->build();

        expect($page->items)->toMatchArray($expectedItems);
        expect($page->pagination)->toMatchArray([
            "offset" => 0,
            "limit" => $limit,
            "total" => $total,
            "type" => "offset",
        ]);
    }

    public function testLimitReturnEmptyArray()
    {
        $limit = 0;
        $request = $this->ofbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $builder = new OffsetPageBuilder($queryBuilderMock, $request);

        $page = $builder->build();

        expect($page->items)->toMatchArray([]);
        expect($page->pagination)->toMatchArray([
            "offset" => 0,
            "limit" => $limit,
            "total" => 0,
            "type" => "offset",
        ]);
    }

    public function testNegativeLimit()
    {
        $limit = -1;
        $request = $this->ofbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $expectedItems = [1, 2, 3, 4];
        $queryBuilderMock->shouldReceive('get')->andReturn(new Collection($expectedItems));
        $builder = new OffsetPageBuilder($queryBuilderMock, $request);

        $page = $builder->build();

        expect($page->items)->toMatchArray($expectedItems);
        expect($page->pagination)->toMatchArray([
            "offset" => 0,
            "limit" => $limit,
            "total" => count($expectedItems),
            "type" => "offset",
        ]);
    }

    public function testNegativeLimitAndForbidToBypassPagination()
    {
        $limit = -1;
        $request = $this->ofbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $builder = new OffsetPageBuilder($queryBuilderMock, $request);

        $page = $builder->forbidToBypassPagination()->build();

        expect($page->items)->toMatchArray([]);
        expect($page->pagination)->toMatchArray([
            "offset" => 0,
            "limit" => $limit,
            "total" => 0,
            "type" => "offset",
        ]);
    }

    public function testPositiveLimitCannotExceedMaxLimit()
    {
        $limit = 10;
        $maxLimit = 5;
        $request = $this->ofbMakeRequest([
            'pagination' => [
                'limit' => $limit,
            ],
        ]);
        $queryBuilderMock = $this->mockQueryBuilder();
        $expectedItems = [1, 2, 3, 4];
        $queryBuilderMock->shouldReceive('clone')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('skip')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('limit')->andReturn($queryBuilderMock);
        $queryBuilderMock->shouldReceive('get')->andReturn(new Collection($expectedItems));

        $builder = new OffsetPageBuilder($queryBuilderMock, $request);

        $page = $builder->maxLimit($maxLimit)->build();

        expect($page->items)->toMatchArray($expectedItems);
        expect($page->pagination)->toMatchArray([
            "offset" => 0,
            "limit" => $maxLimit,
            "total" => count($expectedItems),
            "type" => "offset",
        ]);
    }
}
